clear all
close all
clc

if isempty(gcp('nocreate'))
    parpool(5);
end

addpath('Mechanics');
addpath('Chemicals');
addpath('KMG1.0');

nsim = 5000;
for simulation = 4906:nsim
    disp(['Simulation ' ,num2str(simulation),'.']);
    disp('Day:');

    % Set variation
    P = Parameters;

    % Mesh
    load('basis.mat');

    % Initial conditions
    Vars = InitialConditions(n,geo,p{1},P,a,s);

    tStart = tic;

    while day(it) < 365

        % Initialize new solutions for variables
        Vars_tdt = Vars{it};

        % Solve the equations using inner Picard iterations and the
        % FCT-algorithm
        max_norm = Inf; iter = 0;
        while max_norm > 5e-2 && iter < 6

            Mass_tdt = Mass(n, elmat{it}, p_tdt, Ind);

            % Solve for constituents (using FCT)
            [Vars_tdt(:,1:4), max_norm] = SolveChemicals(n, elmat{it}, EssBnd, ...
                p_tdt, p_half, Vars_tdt(:,1:4), Vars{it}(:,1:4), P, Mass_tdt, ...
                Mass_t, dt);

            % Solve for mechanics
            [Vars_tdt(:,5:9), max_norm1] = SolveMechanics(n, elmat{it}, EssBnd, ...
                HorzBnd, VertBnd, p_tdt, Vars_tdt(:,1:9), ...
                Vars{it}(:,5:9), P, Mass_tdt, Mass_t, dt);

            % Compute max_norm and update variables and Pic
            max_norm = max([max_norm, max_norm1]);

            % Compute displacements
            Vars_tdt(:,10) = Vars_tdt(:,10) + dt*Vars_tdt(:,5); % u_tdt
            Vars_tdt(:,11) = Vars_tdt(:,11) + dt*Vars_tdt(:,6); % v_tdt

            % Update the mesh
            p_tdt(1,:) = p{it}(1,:) + dt*Vars_tdt(:,5)';
            p_tdt(2,:) = p{it}(2,:) + dt*Vars_tdt(:,6)';

            p_half(1,:) = p{it}(1,:) + 0.5*dt*Vars_tdt(:,5)';
            p_half(2,:) = p{it}(2,:) + 0.5*dt*Vars_tdt(:,6)';

            [~,qt,~] = tarea(p_tdt',elmat{it}); q_min = min(qt);
            if q_min < 0.5                
                Remeshing = [Remeshing,day(it)];
                tStart_remesh = tic;
                [p{it}, elmat{it}, Vars{it}, EssBnd, HorzBnd, VertBnd] = ...
                    remesh_global_domain(s, h0, p{it}, WoundBnd{it}, Vars{it}, geo);
                Remeshing_time = [Remeshing_time,toc(tStart_remesh)];

                Vars_tdt = Vars{it};
                p_tdt = p{it}; p_half = p_tdt;
                n = size(Vars{it},1);
                Ind = sub2ind([n,n],elmat{it},elmat{it});
                Mass_t = Mass(n, elmat{it}, p{it}, Ind);
                iter = -1; max_norm = Inf;
            end

            iter = iter + 1;
        end

        % Continue if max_norm <= 5e-2, otherwise restart
        if max_norm <= 5e-2            

            DT(it) = dt;
            disp(day(it));

            it = it + 1;
            day(it) = day(it-1) + dt; %#ok<*SAGROW>

            % Estimate wound boundary
            V1 = Interpolate_new_mesh(WoundBnd{it-1}', p_tdt, Vars_tdt(:,5:6));
            WoundBnd(it) = {WoundBnd{it-1} + dt*V1};
            Pgon = polyshape([0,0; WoundBnd{it}],'Simplify',false);
            RDA(it) = Pgon.area/Area_init;

            % Strain energy
            fun = 2/(1-P(24)^2)*(P(23)*Vars_tdt(:,4).^(0.5).*(Vars_tdt(:,7).^2 + ...
                2*P(24)*Vars_tdt(:,7).*Vars_tdt(:,9) + 1/(1+P(24))*Vars_tdt(:,8).^2 + ...
                Vars_tdt(:,9).^2));

            x = unique(round(p_tdt(1,:),4));
            y = unique(round(p_tdt(2,:),4));
           
            [X,Y] = ndgrid(x,y);           
            F = scatteredInterpolant(p_tdt(1,:)', p_tdt(2,:)', fun);
            Z = F(X,Y);

            TSE(it) = trapz(x,trapz(y,Z,2));

            % Update variables
            Vars(it) = {Vars_tdt};
            p(it) = {p_tdt};
            elmat(it) = elmat(it-1);

            dt = min(max_dt, 1.1*dt);

            if RDA(it) - RDA(it-1) > 0 && ...
                    (RDA(it) - RDA(it-1))*100/(day(it) - day(it-1)) < 0.10
                max_dt = max_dt1;
            end
            if RDA(it) - RDA(it-1) > 0 && ...
                    (RDA(it) - RDA(it-1))*100/(day(it) - day(it-1)) < 0.01
                max_dt = max_dt2;
            end
            if day(it) + dt > T, dt = T - day(it); end

            Mass_t = Mass_tdt;
            Quality(it) = q_min;
        else
            dt = 0.8*dt;
        end
    end
    tEnd = toc(tStart);
    fprintf('-----------------------------------------------------\n');
    fprintf('The total simulation time for the simulation is %5.1f minutes.\n',tEnd/60);

    filename = sprintf('%s_%d.mat','Results\sim',simulation);
    save(filename,'P','day','RDA','TSE','WoundBnd','tEnd');
end